/*
 *                                                 Copyright (C) 2002 Chir
 *
 * THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
 * IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
 * THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
 * (AT YOUR OPTION) ANY LATER VERSION.
 *
 * THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL,
 * BUT WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  SEE THE
 * GNU GENERAL PUBLIC LICENSE FOR MORE DETAILS.
 *
 * YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
 * ALONG WITH THIS PROGRAM; IF NOT, WRITE TO THE FREE SOFTWARE
 * FOUNDATION, INC., 59 TEMPLE PLACE, SUITE 330, BOSTON, MA 02111-1307 USA
 */

package org.chir;

import java.io.*;
import java.util.*;

/**
 * Handles the execution of a shell command.
 *
 * @author  Chir
 */
public class CommandLine
{
    /**
     * Executes a shell command.
     * @param cmdT      The command line to run.
     * @param dirF      The directory from which to launch the command.
     * @param channelNum The console channel to read (1=cout, 2=cerr).
     * @param allowOut  False if any output from the command must be reported as an error.
     * @param verbose   True to echo command output to the console.
     * @return          True if command was executed successfully.
     */
    public static boolean ExecuteCommand( String cmdT, File dirF, int channelNum, boolean allowOut, boolean verbose ) throws java.io.IOException
    {
/*        
        // --- Check that the command exists ---
        try
        {
            Process p0 = null;
            p0 = Runtime.getRuntime().exec( cmdT, null, dirF );  // jdk1.3
            p0.destroy();    // kill immediately.
        }
        catch( java.io.IOException e1 )  { throw e1; }          // Command not found.
*/
        // --- Actually execute the command ---
        Process p = null;
        boolean has_output = false;     // It could mean error, for some commands that don't normally output anything.
        try
        {
            p = Runtime.getRuntime().exec( cmdT, null, dirF );  // jdk1.3
            // b = new ProcessBuilder(cmdT);					// jdk1.5
            InputStream is1;
            switch( channelNum )
            {
                case 1 :    is1 = p.getInputStream();			// cout.
                			break;
                default :   is1 = p.getErrorStream();           // cerr.
            }
            byte buf[] = new byte[1000];
            int len;
            while( (len = is1.read(buf,0,buf.length)) != -1 )
            {
                String str = new String( buf, "ISO-8859-1" );
                if( verbose )  System.out.println( "Process out: " + str );
                has_output = true;
                Thread.currentThread().yield();                 // Give a chance to other threads.
            }
            p.waitFor();
            if( verbose )  System.out.println( "return code: " + p.exitValue() );
        }
        catch( java.io.IOException e1 )  { throw e1; }          // Command not found.
        catch( InterruptedException e3 )
        {
//                System.out.println( "InterruptedException! return code: " + p.exitValue() );
            return false;
        }

        return ( p != null  &&  p.exitValue() == 0  &&  ( allowOut  ||  ! has_output) );
    }
    /**
     * Checks if a shell command is recognised.
     * @param cmdT  The command line to run.
     * @return      True if command was executed successfully.
     */
/*
    public static boolean CheckCommand( String cmdT )
    {
        Process p = null;
        
        try
        {
            p = Runtime.getRuntime().exec( cmdT );  // jdk1.3
        }
        catch( java.io.EOFException e1 )  { }
        catch( java.io.IOException e2 )  { }

        return ( p != null );
    }
*/
}
// ----------------------------------------
