/*
 * =======================================================================
 * ============================ WIZARD BEAN ==============================
 * =======================================================================
 *
 *                                                 Copyright (C) 2002 Chir
 *
 * DESCRIPTION:   This Swing bean contains a wizard framework,
 *                to make it easier to create wizards.
 *
 * THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
 * IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
 * THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
 * (AT YOUR OPTION) ANY LATER VERSION.
 *
 * THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL,
 * BUT WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  SEE THE
 * GNU GENERAL PUBLIC LICENSE FOR MORE DETAILS.
 *
 * YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
 * ALONG WITH THIS PROGRAM; IF NOT, WRITE TO THE FREE SOFTWARE
 * FOUNDATION, INC., 59 TEMPLE PLACE, SUITE 330, BOSTON, MA 02111-1307 USA
 */

package org.chir.swing;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.net.*;
import java.util.*;
import java.beans.*;
import javax.swing.*;

/**
 * The Wizard Manager component.
 *
 * @author  Chir
 */
public class WizardComponent extends JPanel implements ActionListener, ComponentListener, KeyListener, PropertyChangeListener, javax.swing.event.DocumentListener, javax.swing.event.ListSelectionListener, javax.swing.event.ChangeListener
{
    /**
     * Creates a new Wizard Manager.
     */
    public WizardComponent()
    {
        initComponents();
    }

    public void setNextText( String s )
    {
        nextButton.setText( s );
        nextButton.setMnemonic( s.charAt(0) );
    }
    public String getNextText()
    {
        return nextButton.getText();
    }

    public void setBackText( String s )
    {
        backButton.setText( s );        
        backButton.setMnemonic( s.charAt(0) );
    }
    public String getBackText()
    {
        return backButton.getText();
    }

    public void setFinishText( String s )
    {
        finishButton.setText( s );        
        finishButton.setMnemonic( s.charAt(0) );
    }
    public String getFinishText()
    {
        return finishButton.getText();
    }
    
    public void addWizardPanel( WizardPanel p )
    {
        p.setParent( this );
        cardPanel.add( p, "card" + m_wizardPanels.size() );
        m_wizardPanels.addElement( p );
        showCurrentCard();
    }
    
    public boolean doMoveNext()
    {
        boolean b = false;

        if( m_currentCard < m_wizardPanels.size() )
        {
            if( b = m_wizardPanels.elementAt(m_currentCard).moveNext() )
            {
                ++m_currentCard;
                showCurrentCard();
            }
        }
        return b;
    }
    
    public boolean doMoveBack()
    {
        boolean b = false;

        if( m_currentCard > 0 )
        {
            if( m_wizardPanels.elementAt(m_currentCard).moveBack() )
            {
                --m_currentCard;
                showCurrentCard();
            }
        }
        return b;
    }

    public void actionPerformed( ActionEvent ev )
    {
        if( ev.getSource() == backButton )
        {
            doMoveBack();
        }
        if( ev.getSource() == nextButton )
        {
            doMoveNext();
        }
        if( ev.getSource() == finishButton )
        {
//            m_wizardPanels.elementAt(m_currentCard)).moveNext();
            doMoveNext();
        }
    }

    public void componentHidden( ComponentEvent e )
    {
        showCurrentCard();
    }
    public void componentMoved( ComponentEvent e )
    {
        showCurrentCard();
    }
    public void componentResized( ComponentEvent e )
    {
        showCurrentCard();
    }
    public void componentShown( ComponentEvent e )
    {
        showCurrentCard();
    }
    
    public void keyPressed( KeyEvent e )
    {
        updateButtons();
    }    
    public void keyReleased( KeyEvent e )
    {
        updateButtons();
    }
    public void keyTyped( KeyEvent e )
    {
        updateButtons();
    }
    
    public void propertyChange( PropertyChangeEvent evt )
    {
        updateButtons();
    }
    
    public void changedUpdate( javax.swing.event.DocumentEvent e )
    {
        updateButtons();
    }
    public void insertUpdate( javax.swing.event.DocumentEvent e )
    {
        updateButtons();
    }
    public void removeUpdate( javax.swing.event.DocumentEvent e )
    {
        updateButtons();
    }
    
    public void valueChanged( javax.swing.event.ListSelectionEvent e )
    {
        updateButtons();
    }

    public void stateChanged( javax.swing.event.ChangeEvent evt )
    {
        updateButtons();
    }

    /** Disable the Next button, to prevent going on while the conditions are not fulfilled. */
    private void showCurrentCard()
    {
        if( m_currentCard < m_wizardPanels.size() )
        {
            updateButtons();
            if( getRootPane() != null )
            {
                getRootPane().setDefaultButton( nextButton.isVisible() ? nextButton : finishButton );
                ( nextButton.isVisible() ? nextButton : finishButton ).requestFocus();
            }
    
            CardLayout l = (CardLayout)(cardPanel.getLayout());
    
//            jPanel1.requestFocus();
            m_wizardPanels.elementAt(m_currentCard).showPanel();    // To set component invisibilities, while off-screen.
            l.show( cardPanel, "card" + m_currentCard );
            m_wizardPanels.elementAt(m_currentCard).showPanel();    // To request focus, once visible.
        }
    }

    /** Disable the Next button, to prevent going on while the conditions are not fulfilled. */
    private void updateButtons()
    {
        if( m_currentCard < m_wizardPanels.size() )
        {
            boolean b;
            
            b = m_wizardPanels.elementAt(m_currentCard).canMoveNext();
            nextButton.setEnabled( b );
            nextButton.setVisible( m_currentCard < m_wizardPanels.size() - 1 ); // Don't show NEXT at all if this is the last card.
            finishButton.setVisible( ! nextButton.isVisible() );
            b = m_wizardPanels.elementAt(m_currentCard).canMoveBack();
            backButton.setVisible( b  &&  m_currentCard > 0 );    // Don't show BACK at all if this is the first card.
        }
    }

    private void initComponents()
    {
//        GridBagConstraints gridBagConstraints;

        setLayout( new BorderLayout() );

        cardPanel.setLayout( new CardLayout() );
/*
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
*/
        add( cardPanel, BorderLayout.NORTH );
/*
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
*/

        JPanel spacerPanel = new JPanel();
        spacerPanel.setMinimumSize( new Dimension(500, 15) );
        spacerPanel.setMaximumSize( new Dimension(500, 15) );
        spacerPanel.setPreferredSize( new Dimension(500, 15) );
        spacerPanel.setLayout( new BorderLayout() );
        add( spacerPanel, BorderLayout.CENTER );

        add( jPanel1, BorderLayout.SOUTH );

        jPanel1.setMinimumSize( new Dimension(500, 25) );
        jPanel1.setPreferredSize( new Dimension(500, 25) );
        jPanel1.setRequestFocusEnabled( false );

        backButton.setIcon( new javax.swing.ImageIcon(getClass().getResource("gui/backward.gif")) );
        backButton.setText( "Back" );
        backButton.setDefaultCapable( false );
//        backButton.setMaximumSize( new Dimension(120, 25) );
        backButton.setMinimumSize( new Dimension(120, 25) );
//        backButton.setPreferredSize( new Dimension(120, 25) );
        backButton.setRequestFocusEnabled( false );
        backButton.addActionListener( this );

        finishButton.setIcon( new javax.swing.ImageIcon(getClass().getResource("gui/stop.gif")) );
        finishButton.setText( "Finish" );
        finishButton.setHorizontalTextPosition(javax.swing.SwingConstants.LEFT);
//        finishButton.setMaximumSize( new Dimension(120, 25) );
        finishButton.setMinimumSize( new Dimension(120, 25) );
//        finishButton.setPreferredSize( new Dimension(120, 25) );
        nextButton.setRequestFocusEnabled( true );
        finishButton.addActionListener( this );

        nextButton.setIcon( new javax.swing.ImageIcon(getClass().getResource("gui/forward.gif")) );
        nextButton.setText( "Next" );
        nextButton.setHorizontalTextPosition(javax.swing.SwingConstants.LEFT);
//        nextButton.setMaximumSize( new Dimension(120, 25) );
        nextButton.setMinimumSize( new Dimension(120, 25) );
//        nextButton.setPreferredSize( new Dimension(120, 25) );
        nextButton.setRequestFocusEnabled( true );
        nextButton.addActionListener( this );

        GridLayout l = new GridLayout(1,3);
        jPanel1.setLayout( l );
        l.setHgap( 75/2 );  l.setVgap( 75 );
        jPanel1.add( backButton  );
        jPanel1.add( finishButton );
        jPanel1.add( nextButton );
/*
        jPanel1.setLayout( new GridBagLayout() );
        jPanel1.setAlignmentX(0.0F);
        jPanel1.setAlignmentY(0.0F);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST; // GridBagConstraints.SOUTHWEST;
        gridBagConstraints.insets = new Insets(20, 6, 6, 6);
        jPanel1.add( backButton, gridBagConstraints );
        
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = GridBagConstraints.CENTER;
        gridBagConstraints.insets = new Insets(20, 6, 6, 6);
        jPanel1.add( finishButton, gridBagConstraints );

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
        gridBagConstraints.gridheight = GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = GridBagConstraints.EAST; //GridBagConstraints.SOUTHEAST;
        gridBagConstraints.insets = new Insets(20, 6, 6, 6);
        jPanel1.add( nextButton, gridBagConstraints );        
//        pack();
*/
        addComponentListener( this );
        jPanel1.addComponentListener( this );
        cardPanel.addComponentListener( this );
    }

    // -----------------------------------------------------------------------
    private JButton finishButton = new JButton();
    private JButton backButton = new JButton();
    private JButton nextButton = new JButton();
    private JPanel  jPanel1 = new JPanel();         // Bottom panel, with Previous&Next buttons.
    private JPanel  cardPanel = new JPanel();       // Card panel, for the wizard pages.
    private Vector<WizardPanel>  m_wizardPanels = new Vector<WizardPanel>();  // All the panels included in the cardPanel.
    private int     m_currentCard = 0;              // Wizard Panel currently shown.
}